{
	"name": "segancha-theme-{{name}}",
	"version": "{{version}}",
	"engines": {
		"vscode": "^0.9.0"
	},
	"publisher": "{{user}}",
	"contributes": {
		"themes": [
			{
				"label": "Segancha Colors Theme {{name}}",
				"uiTheme": "{{ui-theme}}",
				"path": "./themes/segancha-theme.json"
			}
		]
	}
}