{
    "name": "{{name}}",
    "version": "{{version}}",
    "user": "{{user}}",
    "ui-theme": "{{ui-theme}}",

    // Foreground
    "fg-hex": "{{fg-hex}}",

    // Foreground Palette
    "main-0-hex": "{{main-0-hex}}",
    "main-1-hex": "{{main-1-hex}}",
    "main-2-hex": "{{main-2-hex}}",
    "main-3-hex": "{{main-3-hex}}",
    "main-4-hex": "{{main-4-hex}}",
    "main-5-hex": "{{main-5-hex}}",
    "main-6-hex": "{{main-6-hex}}",

    // Lines
    "line-2-hex": "{{line-2-hex}}",
    "line-1-hex": "{{line-1-hex}}",
    "line-0-hex": "{{line-0-hex}}",

    // Semantic Foregrounds
    "red-2-hex": "{{red-2-hex}}",
    "yellow-2-hex": "{{yellow-2-hex}}",
    "green-2-hex": "{{green-2-hex}}",
    "blue-2-hex": "{{blue-2-hex}}",

    // Semantic Inversed Backgrounds
    "red-1-hex": "{{red-1-hex}}",
    "yellow-1-hex": "{{yellow-1-hex}}",
    "green-1-hex": "{{green-1-hex}}",
    "blue-1-hex": "{{blue-1-hex}}",

    // Semantic Backgrounds/Inversed Foregrounds
    "red-0-hex": "{{red-0-hex}}",
    "yellow-0-hex": "{{yellow-0-hex}}",
    "green-0-hex": "{{green-0-hex}}",
    "blue-0-hex": "{{blue-0-hex}}",

    // Backgrounds
    "bg-2-hex": "{{bg-2-hex}}",
    "bg-1-hex": "{{bg-1-hex}}",
    "bg-0-hex": "{{bg-0-hex}}"
}