{
    "name": "Segancha Colors Theme",
    "colors": {
        // Contrast colors (This is not a high contrast theme)
        // "contrastActiveBorder": "#f00",
        // "contrastBorder": "#f00",

        // Base colors
        // "focusBorder": "#f00",
        "foreground": "#{{fg-hex}}",
        "widget.shadow": "#00000033",
        "selection.background": "#{{yellow-0-hex}}",
        "errorForeground": "#{{red-2-hex}}",

        // Button control
        "button.background": "#{{blue-2-hex}}",
        "button.foreground": "#{{bg-0-hex}}",
        "button.hoverBackground": "#{{blue-2-hex}}cc",

        // Dropdown control
        "dropdown.background": "#{{bg-1-hex}}",
        "dropdown.border": "#{{blue-0-hex}}",
        "dropdown.foreground": "#{{fg-hex}}",

        // Input control
        "input.background": "#{{bg-0-hex}}",
        "input.border": "#{{blue-0-hex}}",
        "input.foreground": "#{{fg-hex}}",
        "input.placeholderForeground": "#{{line-1-hex}}",
        "inputOption.activeBorder": "#{{blue-2-hex}}",
        "inputValidation.errorBackground": "#{{red-0-hex}}",
        "inputValidation.errorBorder": "#{{red-2-hex}}",
        "inputValidation.infoBackground": "#{{blue-0-hex}}",
        "inputValidation.infoBorder": "#{{blue-2-hex}}",
        "inputValidation.warningBackground": "#{{yellow-0-hex}}",
        "inputValidation.warningBorder": "#{{yellow-2-hex}}",

        // Scroll bar control
        "scrollbar.shadow": "#00000033",
        "scrollbarSlider.activeBackground": "#{{blue-2-hex}}44",
        "scrollbarSlider.background": "#{{fg-hex}}11",
        "scrollbarSlider.hoverBackground": "#{{blue-2-hex}}22",

        // Badge
        "badge.background": "#{{blue-2-hex}}",
        "badge.foreground": "#{{bg-0-hex}}",

        // Progress bar
        "progressBar.background": "#{{green-1-hex}}",

        // Lists and trees
        "list.activeSelectionBackground": "#{{bg-2-hex}}",
        "list.activeSelectionForeground": "#{{fg-hex}}",
        "list.dropBackground": "#{{blue-0-hex}}",
        "list.focusBackground": "#{{blue-0-hex}}",
        "list.focusForeground": "#{{fg-hex}}",
        "list.highlightForeground": "#{{yellow-2-hex}}",
        "list.hoverBackground": "#{{bg-2-hex}}",
        "list.hoverForeground": "#{{fg-hex}}",
        "list.inactiveSelectionBackground": "#{{bg-2-hex}}",
        "list.inactiveSelectionForeground": "#{{fg-hex}}",

        // Activity bar
        "activityBar.background": "#{{bg-0-hex}}",
        "activityBar.border": "#{{line-0-hex}}",
        "activityBar.dropBackground": "#{{blue-0-hex}}",
        "activityBar.foreground": "#{{fg-hex}}",
        "activityBarBadge.background": "#{{blue-2-hex}}",
        "activityBarBadge.foreground": "#{{bg-0-hex}}",

        // Side bar
        "sideBar.background": "#{{bg-1-hex}}",
        "sideBar.border": "#{{line-0-hex}}",
        "sideBar.foreground": "#{{line-2-hex}}",
        "sideBarSectionHeader.background": "#{{bg-0-hex}}",
        "sideBarSectionHeader.foreground": "#{{line-2-hex}}",
        "sideBarTitle.foreground": "#{{line-2-hex}}",

        // Editor groups and tabs
        "editorGroup.background": "#{{bg-2-hex}}",
        "editorGroup.border": "#{{line-0-hex}}",
        "editorGroup.dropBackground": "#{{bg-1-hex}}",
        "editorGroupHeader.noTabsBackground": "#{{bg-1-hex}}",
        "editorGroupHeader.tabsBackground": "#{{bg-1-hex}}",
        "editorGroupHeader.tabsBorder": "#{{line-0-hex}}",
        "tab.activeBackground": "#{{bg-0-hex}}",
        "tab.activeForeground": "#{{blue-2-hex}}",
        "tab.border": "#{{line-0-hex}}",
        "tab.inactiveBackground": "#{{bg-1-hex}}",
        "tab.inactiveForeground": "#{{line-2-hex}}",
        "tab.unfocusedActiveForeground": "#{{line-2-hex}}",
        "tab.unfocusedInactiveForeground": "#{{line-2-hex}}",

        // Editor colors

        "editor.background": "#{{bg-0-hex}}",
        "editor.foreground": "#{{fg-hex}}",
        "editorLineNumber.foreground": "#{{line-0-hex}}",
        "editorCursor.foreground": "#{{red-2-hex}}",
        "editor.selectionBackground": "#{{yellow-0-hex}}",
        "editor.selectionHighlightBackground": "#{{yellow-0-hex}}",
        "editor.inactiveSelectionBackground": "#{{bg-1-hex}}",
        "editor.wordHighlightBackground": "#{{blue-0-hex}}",
        "editor.wordHighlightStrongBackground": "#{{green-0-hex}}",
        "editor.findMatchBackground": "#{{yellow-0-hex}}",
        "editor.findMatchHighlightBackground": "#{{blue-0-hex}}",
        "editor.findRangeHighlightBackground": "#{{yellow-0-hex}}33",
        "editor.hoverHighlightBackground": "#{{bg-2-hex}}",
        "editor.lineHighlightBackground": "#{{bg-1-hex}}",
        // "editor.lineHighlightBorder": "#f00",
        "editorLink.activeForeground": "#{{main-5-hex}}",
        "editor.rangeHighlightBackground": "#{{yellow-0-hex}}33",
        "editorWhitespace.foreground": "#{{line-0-hex}}",
        "editorIndentGuide.background": "#{{line-0-hex}}",
        "editorRuler.foreground": "#{{line-1-hex}}",
        "editorCodeLens.foreground": "#{{line-1-hex}}",
        "editorBracketMatch.background": "#{{green-0-hex}}",
        "editorBracketMatch.border": "#{{green-2-hex}}",
        "editorOverviewRuler.border": "#{{line-0-hex}}",
        "editorOverviewRuler.findMatchForeground": "#{{yellow-1-hex}}",
        "editorOverviewRuler.rangeHighlightForeground": "#{{yellow-1-hex}}",
        "editorOverviewRuler.selectionHighlightForeground": "#{{blue-1-hex}}",
        "editorOverviewRuler.wordHighlightForeground": "#{{blue-1-hex}}",
        "editorOverviewRuler.wordHighlightStrongForeground": "#{{green-1-hex}}",
        "editorOverviewRuler.modifiedForeground": "#{{yellow-1-hex}}",
        "editorOverviewRuler.addedForeground": "#{{green-1-hex}}",
        "editorOverviewRuler.deletedForeground": "#{{red-1-hex}}",
        "editorOverviewRuler.errorForeground": "#{{red-1-hex}}",
        "editorOverviewRuler.warningForeground": "#{{red-1-hex}}",
        "editorOverviewRuler.infoForeground": "#{{blue-1-hex}}",
        // "editorError.border": "#f0f",
        "editorError.foreground": "#{{red-2-hex}}",
        "editorWarning.foreground": "#{{yellow-2-hex}}",
        // "editorWarning.border": "#f00", 
        "editorGutter.addedBackground": "#{{green-1-hex}}",
        "editorGutter.background": "#{{bg-0-hex}}",
        "editorGutter.deletedBackground": "#{{red-1-hex}}",
        "editorGutter.modifiedBackground": "#{{yellow-1-hex}}",

        // Diff editor colors
        "diffEditor.insertedTextBackground": "#{{green-1-hex}}20",
        // "diffEditor.insertedTextBorder": "#{{green-2-hex}}",
        "diffEditor.removedTextBackground": "#{{red-1-hex}}20",
        // "diffEditor.removedTextBorder": "#{{green-2-hex}}",

        // Editor widget colors
        "editorWidget.background": "#{{bg-1-hex}}",
        "editorWidget.border": "#{{line-0-hex}}",
        "editorSuggestWidget.background": "#{{bg-1-hex}}",
        "editorSuggestWidget.border": "#{{fg-hex}}",
        "editorSuggestWidget.foreground": "#{{fg-hex}}",
        "editorSuggestWidget.highlightForeground": "#{{fg-hex}}",
        "editorSuggestWidget.selectedBackground": "#{{green-1-hex}}",
        "editorHoverWidget.background": "#{{bg-0-hex}}",
        "editorHoverWidget.border": "#{{fg-hex}}",
        "debugExceptionWidget.background": "#{{red-0-hex}}",
        "debugExceptionWidget.border": "#f{{red-2-hex}}",
        "editorMarkerNavigation.background": "#{{bg-1-hex}}",
        "editorMarkerNavigationError.background": "#{{red-0-hex}}",
        "editorMarkerNavigationWarning.background": "#{{yellow-0-hex}}",
        "editorMarkerNavigationInfo.background": "#{{blue-0-hex}}",

        // Peek view colors
        "peekView.border": "#{{green-2-hex}}",
        "peekViewEditor.background": "#{{bg-0-hex}}",
        "peekViewEditor.matchHighlightBackground": "#{{yellow-0-hex}}",
        "peekViewEditorGutter.background": "#{{bg-1-hex}}",
        "peekViewResult.background": "#{{bg-0-hex}}",
        "peekViewResult.fileForeground": "#{{fg-hex}}",
        "peekViewResult.lineForeground": "#{{line-2-hex}}",
        "peekViewResult.matchHighlightBackground": "#{{green-0-hex}}",
        "peekViewResult.selectionBackground": "#{{green-1-hex}}",
        "peekViewResult.selectionForeground": "#{{fg-hex}}",
        "peekViewTitle.background": "#{{green-0-hex}}",
        "peekViewTitleDescription.foreground": "#{{line-2-hex}}",
        "peekViewTitleLabel.foreground": "#{{fg-hex}}",

        // Merge conflicts
        // "merge.border": "#f00",
        "merge.currentContentBackground": "#{{blue-0-hex}}",
        "merge.currentHeaderBackground": "#{{blue-1-hex}}40",
        "merge.incomingContentBackground": "#{{yellow-0-hex}}60",
        "merge.incomingHeaderBackground": "#{{yellow-1-hex}}60",
        "editorOverviewRuler.currentContentForeground": "#{{blue-2-hex}}",
        "editorOverviewRuler.incomingContentForeground": "#{{yellow-2-hex}}",

        // Panel colors
        "panel.background": "#{{bg-0-hex}}",
        "panel.border": "#{{line-0-hex}}",
        "panelTitle.activeBorder": "#{{blue-2-hex}}",
        "panelTitle.activeForeground": "#{{blue-2-hex}}",
        "panelTitle.inactiveForeground": "#{{line-2-hex}}",

        // Status bar colors
        "statusBar.background": "#{{bg-2-hex}}",
        // "statusBar.border": "#{{line-0-hex}}",
        "statusBar.debuggingBackground": "#{{yellow-2-hex}}",
        "statusBar.debuggingForeground": "#{{bg-0-hex}}",
        "statusBar.foreground": "#{{fg-hex}}",
        "statusBar.noFolderBackground": "#{{bg-2-hex}}",
        "statusBar.noFolderForeground": "#{{line-2-hex}}",
        "statusBarItem.activeBackground": "#{{bg-1-hex}}",
        "statusBarItem.hoverBackground": "#{{bg-1-hex}}",
        "statusBarItem.prominentBackground": "#{{green-1-hex}}",
        "statusBarItem.prominentHoverBackground": "#{{green-1-hex}}cc",

        // Title bar colors (macOS)
        "titleBar.activeBackground": "#{{bg-2-hex}}",
        "titleBar.activeForelround": "#{{line-2-hex}}",
        "titleBar.inactiveBackground": "#{{bg-2-hex}}",
        "titleBar.inactiveForeground": "#{{line-0-hex}}",

        // Notification dialog colors
        "notification.background": "#{{bg-1-hex}}",
        "notification.foreground": "#{{line-2-hex}}",
        "notification.buttonBackground": "#{{blue-2-hex}}",
        "notification.buttonHoverBackground": "#{{blue-2-hex}}cc",
        "notification.buttonForeground": "#{{bg-0-hex}}",
        "notification.infoBackground": "#{{blue-2-hex}}",
        "notification.infoForeground": "#{{blue-0-hex}}",
        "notification.warningBackground": "#{{yellow-2-hex}}",
        "notification.warningForeground": "#{{yellow-0-hex}}",
        "notification.errorBackground": "#{{red-2-hex}}",
        "notification.errorForeground": "#{{red-0-hex}}",

        // Extensions
        "extensionButton.prominentBackground": "#{{green-2-hex}}",
        "extensionButton.prominentForeground": "#{{bg-0-hex}}",
        "extensionButton.prominentHoverBackground": "#{{green-2-hex}}cc",

        // Quick picker
        // "pickerGroup.border": "#f00",
        "pickerGroup.foreground": "#{{blue-2-hex}}",

        // Integrated terminal colors
        "terminal.background": "#{{bg-0-hex}}",
        "terminal.foreground": "#{{line-2-hex}}",
        "terminal.ansiBlack": "#{{line-2-hex}}",
        "terminal.ansiBrightBlack": "#{{fg-hex}}",
        "terminal.ansiRed": "#{{main-0-hex}}",
        "terminal.ansiBrightRed": "#{{main-0-hex}}",
        "terminal.ansiYellow": "#{{main-2-hex}}",
        "terminal.ansiBrightYellow": "#{{main-2-hex}}",
        "terminal.ansiGreen": "#{{main-3-hex}}",
        "terminal.ansiBrightGreen": "#{{main-3-hex}}",
        "terminal.ansiCyan": "#{{main-4-hex}}",
        "terminal.ansiBrightCyan": "#{{main-4-hex}}",
        "terminal.ansiBlue": "#{{main-5-hex}}",
        "terminal.ansiBrightBlue": "#{{main-5-hex}}",
        "terminal.ansiMagenta": "#{{main-6-hex}}",
        "terminal.ansiBrightMagenta": "#{{main-6-hex}}",
        "terminal.ansiWhite": "#{{line-0-hex}}",
        "terminal.ansiBrightWhite": "#{{line-0-hex}}",
        
        // Debug
        "debugToolBar.background": "#{{yellow-1-hex}}",

        // Welcome page
        "welcomePage.buttonBackground": "#{{blue-2-hex}}",
        "welcomePage.buttonHoverBackground": "#{{blue-2-hex}}cc",
        "walkThrough.embeddedEditorBackground": "#{{bg-0-hex}}",

        // Git Colors
        "gitDecoration.modifiedResourceForeground": "#{{yellow-2-hex}}",
        "gitDecoration.deletedResourceForeground": "#{{line-1-hex}}",
        "gitDecoration.untrackedResourceForeground": "#{{green-2-hex}}",
        "gitDecoration.ignoredResourceForeground": "#{{line-1-hex}}",
        "gitDecoration.conflictingResourceForeground": "#{{red-2-hex}}",

        // Unknown
        "descriptionForeground": "#{{line-2-hex}}"
        // "textBlockQuote.background": "#{{fg-hex}}",
        // "textBlockQuote.border": "#{{fg-hex}}",
        // "textCodeBlock.background": "#{{fg-hex}}",
        // "textLink.activeForeground": "#{{fg-hex}}",
        // "textLink.foreground": "#{{fg-hex}}",
        // "textPreformat.foreground": "#{{fg-hex}}",
        // "textSeparator.foreground": "#f0f"
    },
    "tokenColors": [
        {
            "name": "Section",
            "scope": [
                "punctuation.section",
                "punctuation.separator"
            ],
            "settings": {
                "foreground": "#{{main-4-hex}}"
            }
        },
        {
            "name": "Keyword",
            "scope": [
                "keyword"
            ],
            "settings": {
                "foreground": "#{{main-5-hex}}"
            }
        },
        {
            "name": "Constant",
            "scope": [
                "constant",
                "support.constant"
            ],
            "settings": {
                "foreground": "#{{main-2-hex}}"
            }
        },
        {
            "name": "String",
            "scope": [
                "string",
                "punctuation.definition.string"
            ],
            "settings": {
                "foreground": "#{{main-3-hex}}"
            }
        },
        {
            "name": "Entity",
            "scope": [
                "entity",
                "support.function",
                "meta.function",
                "meta.at-rule",
                "meta.definition"
            ],
            "settings": {
                "foreground": "#{{main-0-hex}}"
            }
        },
        {
            "name": "Variable",
            "scope": [
                "variable",
                "support.variable"
            ],
            "settings": {
                "foreground": "#{{main-1-hex}}"
            }
        },
        {
            "name": "Storage & Operator",
            "scope": [
                "keyword.operator",  // different from parent rule 'keyword'
                "storage",
                "support.type",
                "support.class",
                "meta.type"
            ],
            "settings": {
                "foreground": "#{{main-6-hex}}"
            }
        },
        {
            "name": "Comment",
            "scope": [
                "comment",
                "punctuation.definition.comment"
            ],
            "settings": {
                "fontStyle": "italic",
                "foreground": "#{{line-1-hex}}"
            }
        },
        {
            "name": "Invalid",
            "scope": [
                "invalid"
            ],
            "settings": {
                "foreground": "#{{main-0-hex}}"
            }
        }
    ]
}
